package de.kif.backend.model

import de.kif.backend.database.DbWorkGroup
import de.kif.backend.database.DbWorkGroupConstraint
import de.kif.backend.database.dbQuery
import org.jetbrains.exposed.sql.*

class WorkGroup(
    var id: Int = -1,
    var name: String = "",
    var interested: Int = 0,
    var trackId: Int? = null,
    var projector: Boolean = false,
    var resolution: Boolean = false,
    var length: Int = 0,
    var start: Long? = null,
    var end: Long? = null
) {
    var constraints: Set<WorkGroupConstraint> = emptySet()

    suspend fun save() {
        if (id < 0) {
            dbQuery {
                val newId = DbWorkGroup.insert {
                    it[name] = this@WorkGroup.name
                    it[interested] = this@WorkGroup.interested
                    it[trackId] = this@WorkGroup.trackId
                    it[projector] = this@WorkGroup.projector
                    it[resolution] = this@WorkGroup.resolution
                    it[length] = this@WorkGroup.length
                    it[start] = this@WorkGroup.start
                    it[end] = this@WorkGroup.end
                }[DbWorkGroup.id]!!
                this@WorkGroup.id = newId
            }
            for (constraint in constraints) {
                constraint.save(this@WorkGroup.id)
            }
        } else {
            dbQuery {
                DbWorkGroup.update({ DbWorkGroup.id eq id }) {
                    it[name] = this@WorkGroup.name
                    it[interested] = this@WorkGroup.interested
                    it[trackId] = this@WorkGroup.trackId
                    it[projector] = this@WorkGroup.projector
                    it[resolution] = this@WorkGroup.resolution
                    it[length] = this@WorkGroup.length
                    it[start] = this@WorkGroup.start
                    it[end] = this@WorkGroup.end
                }

                DbWorkGroupConstraint.deleteWhere { DbWorkGroupConstraint.workGroupId eq id }
            }
            for (constraint in constraints) {
                constraint.save(this@WorkGroup.id)
            }
        }
    }

    suspend fun delete() {
        val id = id
        if (id >= 0) {
            dbQuery {
                DbWorkGroupConstraint.deleteWhere { DbWorkGroupConstraint.workGroupId eq id }
                DbWorkGroup.deleteWhere { DbWorkGroup.id eq id }
            }
        }
    }

    suspend fun loadConstraints() {
        if (id >= 0) {
            constraints = WorkGroupConstraint.get(id)
        }
    }

    companion object {
        suspend fun get(workGroupId: Int): WorkGroup? = dbQuery {
            val result = DbWorkGroup.select { DbWorkGroup.id eq workGroupId }.firstOrNull() ?: return@dbQuery null
            WorkGroup(
                result[DbWorkGroup.id],
                result[DbWorkGroup.name],
                result[DbWorkGroup.interested],
                result[DbWorkGroup.trackId],
                result[DbWorkGroup.projector],
                result[DbWorkGroup.resolution],
                result[DbWorkGroup.length],
                result[DbWorkGroup.start],
                result[DbWorkGroup.end]
            )
        }?.apply { loadConstraints() }

        suspend fun list(): List<WorkGroup> = dbQuery {
            val query = DbWorkGroup.selectAll()
            query.map { result ->
                WorkGroup(
                    result[DbWorkGroup.id],
                    result[DbWorkGroup.name],
                    result[DbWorkGroup.interested],
                    result[DbWorkGroup.trackId],
                    result[DbWorkGroup.projector],
                    result[DbWorkGroup.resolution],
                    result[DbWorkGroup.length],
                    result[DbWorkGroup.start],
                    result[DbWorkGroup.end]
                )
            }
        }.onEach { it.loadConstraints() }
    }
}
